% Title: Radiation Induced Dark Current and RTS Model
% Creation Date: 13 May 2023
% Last Upate: 26 OCtober 2023
% Authors: Vincent Goiffon (vincent.goiffon@isae-supaero.fr)
% Part of this code is based on the contribution of Alexandre Le Roch
%
% Description: Matlab Implementation of Displacement damage induced Dark
% Current and Dark Current Random Telegraph Models in Silicon Pixel Arrays
%
% NOTE: The DC-RTS model is not yet implemented (work in progress)
%
% The Dark Current Model description can be found in:
% A. Le Roch et al., "Radiation-Induced Leakage Current and Electric Field Enhancement in CMOS Image Sensor Sense Node Floating Diffusions," in IEEE Transactions on Nuclear Science, vol. 66, no. 3, pp. 616-624, March 2019, doi: 10.1109/TNS.2019.2892645.
% Jean-Marc Belloir, Vincent Goiffon, Cédric Virmontois, Mélanie Raine, Philippe Paillet, Olivier Duhamel, Marc Gaillardin, Romain Molina, Pierre Magnan, and Olivier Gilard, "Pixel pitch and particle energy influence on the dark current distribution of neutron irradiated CMOS image sensors," Opt. Express 24, 4299-4315 (2016)
%
% The DC-RTS Model details can be found in:
% Goiffon, Vincent Radiation Effects on CMOS Active Pixel Image Sensors. In: Ionizing Radiation Effects in Electronics. CRC Press, pp. 295-332.
%
% The parameters are set to the empirically determined values for Silicon
% but values determined on other materials can be used to simulate
% radiation effects on other detector technologies (e.g. IR detectors)
%


%% Experimental parameters

% --------------------------------
% Physical constants
% --------------------------------
q = 1.6e-19; %C, elementary charge
k = 1.38e-23; % J/K, Boltzmann constant

%---------------------------------
% Empirical Dark Current Constants
%---------------------------------
% Models Constants
Kdark_Srour = 1.9e5 ; %e-/cm3/sec per MeV/g at 300 K 1 week after irradiation
% Source: J. R. Srour and D. H. Lo, "Universal damage factor for radiation-induced dark current in silicon devices," in IEEE Transactions on Nuclear Science, vol. 47, no. 6, pp. 2451-2459, Dec. 2000, doi: 10.1109/23.903792.
% Dark Current Activation Energy
Kdark_22C_6weeks = 0.0971794254848597; % e-/s/µm3(TeV/g) , Kdark extrapolated to 22C and 6 weeks
EactDC = 0.63; % eV, Activation Energy Ea, typical signature of radiation induced midgap like SRH generation centers
nuDark_22C_6weeks = 4100; %e-/s after 6 weeks and at 22°C depends only on measurement temperature (activation energy ~0.63eV) and annealing time
gammaDark = Kdark_22C_6weeks/nuDark_22C_6weeks; % 1/µm3/(TeV/g) assumed independent of temperature and annealing

%---------------------------------
% Empirical DC RTS Constants !!!!!!!! NOT USED IN THIS VERSION !!!!!!!!!!
%---------------------------------
% DC-RTS Activation Energy
EactDCRTS = 0.6; % eV, typical signature of DC-RTS with a +/-0.3 experimental uncertainty

%---------------------------------
% Experimental inputs (user inputs)
%---------------------------------
% Detector Parameters
DepletionVolume = 10*10*1; % in µm3
NumberOfRows = 1000; % Number of rows in the image/in the pixel array
NumberOfColumns = 1000; % Number of columnes in the image/in the pixel array
IntegrationTime = 1; % Integration Time in s
CVF = 50e-6; % Charge to voltage conversion factor in V/e-.
%readnoise = 200e-6/CVF; %e-rms assuming a readout noise at the analog output of 200 µV rms
FWC = 1/CVF; %in e- assuming a 1V voltage swing (or ADC input range)

% Displacement Damage Dose in TeV/g
% 200 TeV/g <=> 10^11 neq/cm² (1 MeV neutron equivalent fluence) <=> 3x10^-14 J/K
Dd = 200; % TeV/g

% Temperature in K
% For now, temperature and annealing influence is not fully implemented.
% Only use T = 22°C and AnnealingTime = 6 weeks for comparing to
% experimental results
T = 273.15 + 22; % in K

% Time after the end of the irradiation (= room temperature annealing time)
AnnealingTime = 6*7*24*3600; % 6 weeks expressed in second
%=================================================
%% Determination of the Dark Current Distribution
% ================================================

%---------------------------
% Damage Factors calculation
%---------------------------

% (BETA) Room Temperature AnnealingFactor Estimation:
% Linear fit of Srour & Lo annealing factor (TNS 2000) in the 10^4 - 5.10^6 s range
% Normalized to the value @1 week: y = [0.3965 x log10(X) + 3.5948]/1.3024
AnnealingFactor = (-0.3965 * log10(AnnealingTime) + 3.5948)/1.3024; %(BETA)
OperatingTemperatureCorrection = exp((-EactDC*q/k/T)-(-EactDC*q/k/300)); % Based on the Arrhenius Law

% Kdark damage factor calculation
Kdark = Kdark_Srour*AnnealingFactor*OperatingTemperatureCorrection/(1e4)^3*1e6; % e-/s/µm3(TeV/g) 
nuDark = Kdark/gammaDark; % e-/s, same temperature and annealing dependence than Kdark is assumed

% According to Belloir paper nuDark is about 4100 e-/s at 22°C after 6
% weeks annealing. If the measurement temperature is several degree above
% or below 22°C, a new nuDark value shall be estimated using a typical 0.63eV
% activation energy. If the annealing period duration is very different
% from 6 weeks (i.e. less than a week or more than 3 months), a new nuDark
% value shall be estimated based on existing displacement damage induced
% dark current data available in litterature, such as:
% Srour, J. R., & Lo, D. H. (2000). Universal damage factor for radiation-induced dark current in silicon devices. IEEE Transactions on Nuclear Science, 47(6), 2451-2459.
% Raine, M., Goiffon, V., Paillet, P., Duhamel, O., Girard, S., Gaillardin, M., ... & Magnan, P. (2014). Exploring the kinetics of formation and annealing of single particle displacement damage in microvolumes of silicon. IEEE Transactions on Nuclear Science, 61(6), 2826-2833.

% Mean number of interactions per pixel muDark
muDark = gammaDark * DepletionVolume * Dd; % without unit since it represents the mean number of interactions per pixel

%---------------------------
% Dark Frame Generation
%---------------------------

% Number of interactions per pixel (Monte Carlo method)
% For each pixel, the poisson distribution is used to determine the number
% of interaction that occured in the given pixel

DarkCurrentFrame = zeros(NumberOfRows,NumberOfColumns);
NumberOfInteractionsArray = poissrnd(muDark,NumberOfRows,NumberOfColumns);
NumberOfInteractionsVector = reshape(NumberOfInteractionsArray,1,NumberOfRows*NumberOfColumns);
Max_number_of_convolution = max(NumberOfInteractionsVector);

% For loop to determine the dark current generated in each pixel
% if the poisson distribution gives 0 interaction in a pixel ==> the
% generated dark current is 0
% if the poisson distribution gives 1 interaction in a pixel ==> the
% generated dark current is given as one trial from the exprnd distribution
% if the poisson distribution gives x interaction in a pixel ==>
% the generated dark current is the sum of x random number from the exprnd
% distribution
% 

% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% !!! NOTE !!! For high displacement damage dose, if the max number of
% convolutions becomes too high (e.g. > 100), it is much more efficient to
% consider the distribution Gaussian, to determine the mean value with the
% dark current damage factor Kdark and the standard deviation with the
% Central limit theorem (not implemented yet)
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


for i = 1:Max_number_of_convolution+1
    temp=find(NumberOfInteractionsVector==i); % find the index of each pixel with i events
    temp_darkcurrent=exprnd(nuDark,i,length(temp)); % generate a random dark current value from the physical exponential distribution expected after irradiation
    % if there are more than one interaction per pixel (if i> 1), i random
    % dark current values have to be drawn from the exponential
    % distribution and the i values must be summed to get the final total
    % dark current generated by i events
    if i>1
        temp_darkcurrent = sum(temp_darkcurrent,1);
    end
    DarkCurrentFrame(temp)=temp_darkcurrent; % The determined random dark current values have to be properly assigned to their corresponding pixel in the dark current frame
end

% Conversion from Dark Current to Dark Signal
DarkSignalFrameInElectrons = DarkCurrentFrame.*IntegrationTime; % Integration of the dark current over the integration time to get a dark signal value in electrons

% OPTIONNAL: adding dark current shot noise to the dark signal values in electron
% -------------------------------------------------------------------------------
DarkSignalFrameInElectronsWithShotNoise = round(DarkSignalFrameInElectrons+sqrt(DarkSignalFrameInElectrons).*randn(NumberOfRows,NumberOfColumns)); % Shot noise is added as a Gaussian noise with standard deviation equals the square root of the pixel signal dark signal in electron. Values are rounded to take into account the quantized nature of electrons.
% -------------------------------------------------------------------------------
% END OF OPTIONNAL LINE OF CODE



%% OPTIONNAL: Converting to Volt and Adding read noise
% DarkSignalFrame = DarkSignalFrameInElectrons.*CVF; % values are in Volt (OPTIONAL)
% DarkSignalFrame = round(DarkSignalFrame)+randn(NumberOfRows,NumberOfColumns)*readnoise;

%% For verification purpose: displaying a dark signal frame
figure('units','normalized','outerposition',[0 0 1 1])

% Display of the full size dark image clipped at the saturation charge
% (= maximum value of the colorscale is FWC)
subplot(1,3,1)
imagesc(DarkSignalFrameInElectronsWithShotNoise)
colormap(gray)
axis square
set(gca, 'CLim', [0 FWC])
set(gca,'XTick',[], 'YTick', [])
title('Full Resolution Dark Frame (From 0 to FWC)');


% Display of a reduced size dark image (100 x 100 pixels) clipped at the saturation charge
% (= maximum value of the colorscale is FWC)
subplot(1,3,2)
imagesc(DarkSignalFrameInElectronsWithShotNoise(1:100,1:100));
colormap(gray)
axis square
set(gca, 'CLim', [0 FWC])
set(gca,'XTick',[], 'YTick', [])
title('100x100 Pixel Dark Frame');


% Display of the dark signal distribution
subplot(1,3,3)
hist = histogram(DarkSignalFrameInElectronsWithShotNoise)
set(gca, 'YScale', 'log')
title('Dark Signal Distribution');
xlabel('Dark Signal (e-)')
ylabel('Number of Pixels')
axis square